/* ESP8266_ThingSpeak_POST
 *  
 * Egy DHT22 szenzor adatait küldjük el a ThingSpeak szervernek 
 * beállított időközönként (20 s) az ESP8266 WiFi kapcsolatán keresztül.
 * A ThingSpeak szervernek POST módszerrel adjuk át az adatokat.
 * 
 * A POST módszert az alábbi cikk mutatja be:
 * https://RandomNerdTutorials.com/esp8266-nodemcu-http-post-ifttt-thingspeak-arduino/
 * 
 * Tested with Arduino IDE 1.8.13 and ESP8266 Arduino core 2.7.4
 */

#include <ESP8266WiFi.h>
#include <ESP8266HTTPClient.h>
#include "DHT.h"               // Adafruit DHT library
#include "secrets.h"           // Wifi jelszó, ThingSpeak Write API key

#define DHTPIN          14     // GPIO14 (D5) pin a DHT szenzorhoz
DHT dht(DHTPIN, DHT22);
float t = 0.0;                 // Hőmérséklet
float h = 0.0;                 // Páratartalom
const char* serverName = "http://api.thingspeak.com/update";
unsigned long lastTime;
unsigned long timerDelay = 20000;
String apiKey = String("api_key=") + THINGSPEAK_WRITE_APIKEY;

void setup() {
  Serial.begin(115200);        // Soros port inicialálás
  dht.begin();                 // DHT szenzor indítása
  WiFi.mode(WIFI_STA);         // Station (client) mód
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  Serial.print("Connecting to ");
  Serial.print(WIFI_SSID);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();
  Serial.print("Connected! IP address: ");
  Serial.println(WiFi.localIP());
  lastTime = millis();
}

void loop() {
  if (millis() > lastTime) {                    // Ha letelt az idő ...
    float tnew = dht.readTemperature();         // Hőmérséklet kiolvasása
    if (!isnan(tnew)) t = tnew;
    float hnew = dht.readHumidity();            // Páratartalom kiolvasása
    if (!isnan(hnew)) h = hnew;
    String p1 = String("&field1=") + String(t, 1);
    String p2 = String("&field2=") + String(h, 1);
    Serial.println(p1 + p2);
    if (WiFi.status() == WL_CONNECTED) {        // Ha a WiFi hálózat elérhető
      HTTPClient http;                          // Webkliens példányosítása
      http.begin(serverName);                   // Szerver név megadása
      http.addHeader("Content-Type", "application/x-www-form-urlencoded");
      String httpRequestData = apiKey + p1 + p2;// HTTP request adatai
      int httpResponseCode = http.POST(httpRequestData); // adatok kiküldése
      Serial.print("HTTP Response code: ");     // Válaszkód kiírása
      Serial.println(httpResponseCode);         // Sikeres küldés esetén 200-as kód
      http.end();                               // Erőforrások felszabadítása
    }
    else {
      Serial.println("WiFi Disconnected");
    }
    lastTime += timerDelay;                     // A következő eseményidő előjegyzése
  }
}
